package nographics;

/**
 * A car.
 * 
 * @author unknown
 */
public class Car extends Thread {
    /**
     * Amount that a car moves in its x-coordinate in one unit.
     */
    final static int X_MOVEMENT = 50;

    /**
     * Amount that a car moves in its y-coordinate in one unit.
     */
    final static int Y_MOVEMENT = 50;

    /**
     * Y-coordinate of the middle of the bridge.
     */
    final static int MIDDLE = 100;

    /**
     * Y-coordinate of the upper lane.
     */
    final static int UPPER = 50;

    /**
     * Y-coordinate of the lower lane.
     */
    final static int LOWER = 150;

    /**
     * X-coordinate of the head of the left side of the bridge.
     */
    final static int X_LEFT_HEAD = 250;

    /**
     * X-coordinate of the left of the bridge.
     */
    final static int X_LEFT = 300;

    /**
     * X-coordinate of the right of the bridge.
     */
    final static int X_RIGHT = 550;

    /**
     * X-coordinate of the head of the right side of the bridge.
     */
    final static int X_RIGHT_HEAD = 600;

    /**
     * Minimal distance between cars. 
     */
    final static int MIN_DISTANCE = 100;
    
    /**
     * X-coordinate where to park the cars at the left.
     */
    final static int LEFT_PARKING = 0;
    
    /**
     * X-coordinate where to park the cars at the right.
     */
    final static int RIGHT_PARKING = 850;

    private Colour colour; // colour of this car
    private int x; // x-coordinate of this car
    private int y; // y-coordinate of this car 
    private Car inFront; // the car infront of this car
    private Controller controller; // the controller of this car

    /**
     * Initializes this car.
     * 
     * @param colour colour of this car
     * @param inFront the car in front of this car
     * @param controller the controller of this car
     */
    public Car(Colour colour, Car inFront, Controller controller) {
	this.colour = colour;
	this.inFront = inFront;
	this.controller  = controller;

	switch (this.colour) {
	case RED : 
	    this.x = Math.min(LEFT_PARKING, this.inFront.x - MIN_DISTANCE);
	    this.y = LOWER;
	    break;
	case BLUE :
	    this.x = Math.max(RIGHT_PARKING, this.inFront.x + MIN_DISTANCE);
	    this.y = UPPER;
	    break;
	}
    }
    
    /**
     * Initializes this dummy car.
     * 
     * @param x x-coordinate of this car
     */
    public Car(int x) {
	this.colour = null;
	this.x = x;
	this.y = 0; 
	this.inFront = null;
	this.controller = null;
    }

    /**
     * Moves this car.
     */
    public void move() {
	switch (this.colour) {
	case RED :
	    if (this.inFront.x - this.x > MIN_DISTANCE) {
		this.x += X_MOVEMENT;
		if (this.x >= X_LEFT_HEAD && this.x - X_MOVEMENT < X_LEFT_HEAD) {
		    this.controller.enterLeft();
		} else if (this.x > X_LEFT_HEAD && this.x <= X_LEFT) {
		    if (this.y > MIDDLE) {
			this.y -= Y_MOVEMENT;
		    }
		} else if (this.x > X_RIGHT && this.x <= X_RIGHT_HEAD) {
		    if (this.y < LOWER) {
			this.y += Y_MOVEMENT;
		    }
		} else if (this.x > X_RIGHT_HEAD && this.x - X_MOVEMENT <= X_RIGHT_HEAD) {
		    this.controller.leaveRight();
		}
	    }
	    break;
	case BLUE :
	    if (this.x - this.inFront.x > MIN_DISTANCE) {
		this.x -= X_MOVEMENT;
		if (this.x <= X_RIGHT_HEAD && this.x + X_MOVEMENT > X_RIGHT_HEAD) {
		    this.controller.enterRight();
		} else if (this.x < X_RIGHT_HEAD && this.x >= X_RIGHT) {
		    if (this.y < MIDDLE) {
			this.y += Y_MOVEMENT;
		    }
		} else if (this.x < X_LEFT && this.x >= X_LEFT_HEAD) {
		    if (this.y > UPPER) {
			this.y -= Y_MOVEMENT;
		    }
		} else if (this.x < X_LEFT_HEAD && this.x + X_MOVEMENT >= X_LEFT_HEAD) {
		    this.controller.leaveLeft();
		}
		break;
	    }
	}
    }

    /**
     * Tests whether this car is out of sight.
     * 
     * @return true if this car is out of sight, false otherwise.
     */
    private boolean outOfSight() {
	switch (this.colour) {
	case RED :
	    return this.x > 700;
	case BLUE :
	    return this.x < 150;
	default :
	    return true;
	}
    }

    /**
     * Life of a car.
     */
    public void run() {
	while (!this.outOfSight()) {
	    this.move();
	}

	switch (this.colour) {
	case RED :
	    this.x = RIGHT_PARKING;
	    break;
	case BLUE :
	    this.x = LEFT_PARKING;
	    break;
	}
    }
}
